@@ -33,7 +33,7 @@
  * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
  * THE POSSIBILITY OF SUCH DAMAGES.
  *
- * $Id: if_athvar.h 2104 2007-02-13 05:54:52Z proski $
+ * $Id: if_athvar.h 3352 2008-02-12 06:05:28Z mrenzmann $
  */
 
 /*
@@ -101,9 +101,13 @@ typedef void irqreturn_t;
 #endif /* !defined(IRQ_NONE) */
 
 #ifndef SET_MODULE_OWNER
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,23)
 #define	SET_MODULE_OWNER(dev) do {		\
 	dev->owner = THIS_MODULE;		\
 } while (0)
+#else
+#define SET_MODULE_OWNER(dev) do { } while (0)
+#endif
 #endif
 
 #ifndef SET_NETDEV_DEV
@@ -116,6 +120,11 @@ typedef void irqreturn_t;
 #define ATH_GET_NETDEV_DEV(ndev)	((ndev)->class_dev.dev)
 #endif
 
+#ifndef	NETDEV_TX_OK
+#define	NETDEV_TX_OK	0
+#define	NETDEV_TX_BUSY	1
+#endif
+
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2,4,23)
 static inline struct net_device *_alloc_netdev(int sizeof_priv, const char *mask,
 					       void (*setup)(struct net_device *))
@@ -148,10 +157,9 @@ static inline struct net_device *_alloc_netdev(int sizeof_priv, const char *mask
 #undef alloc_netdev
 #define alloc_netdev(s,m,d) _alloc_netdev(s,m,d)
 
-static inline struct proc_dir_entry *PDE(const struct inode *inode)
-{
-	return (struct proc_dir_entry *)inode->u.generic_ip;
-}
+/* Some vendors backport PDE, so make it a macro here */
+#undef PDE
+#define PDE(inode) ((struct proc_dir_entry *)(inode)->u.generic_ip)
 #endif
 
 
@@ -199,7 +207,9 @@ static inline struct proc_dir_entry *PDE(const struct inode *inode)
 #define	ATH_RXBUF	40		/* number of RX buffers */
 #define	ATH_TXBUF	200		/* number of TX buffers */
 
-#define	ATH_BCBUF	4		/* number of beacon buffers */
+#define ATH_MAXVAPS_MIN 	2	/* minimum number of beacon buffers */
+#define ATH_MAXVAPS_MAX 	64	/* maximum number of beacon buffers */
+#define ATH_MAXVAPS_DEFAULT 	4	/* default number of beacon buffers */
 
 /* free buffer threshold to restart net dev */
 #define	ATH_TXBUF_FREE_THRESHOLD  (ATH_TXBUF / 20) 
@@ -532,38 +542,42 @@ struct ath_softc {
 	struct ath_ratectrl *sc_rc;		/* tx rate control support */
 	struct ath_tx99 *sc_tx99; 		/* tx99 support */
 	void (*sc_setdefantenna)(struct ath_softc *, u_int);
-	unsigned int 	sc_invalid:1,		/* being detached */
-			sc_mrretry:1,		/* multi-rate retry support */
-			sc_softled:1,		/* enable LED gpio status */
-			sc_splitmic:1,		/* split TKIP MIC keys */
-			sc_needmib:1,		/* enable MIB stats intr */
-			sc_hasdiversity:1,	/* rx diversity available */
-			sc_diversity:1, 	/* enable rx diversity */
-			sc_olddiversity:1, 	/* diversity setting before XR enable */	       
-			sc_hasveol:1,		/* tx VEOL support */
-			sc_hastpc:1,		/* per-packet TPC support */
-			sc_dturbo:1,		/* dynamic turbo capable */
-			sc_dturbo_switch:1,	/* turbo switch mode*/
-			sc_dturbo_hold:1,	/* dynamic turbo hold state */
-			sc_rate_recn_state:1,	/* dynamic turbo state recmded by ratectrl */
-			sc_ignore_ar:1,		/* ignore AR during transition */
-			sc_ledstate:1,		/* LED on/off state */
-			sc_blinking:1,		/* LED blink operation active */
-			sc_beacons:1,		/* beacons running */
-			sc_hasbmask:1,		/* bssid mask support */
-			sc_mcastkey:1,		/* mcast key cache search */
-			sc_hastsfadd:1,		/* tsf adjust support */
-			sc_scanning:1,		/* scanning active */
-			sc_nostabeacons:1,	/* no beacons for station */
-			sc_xrgrppoll:1,		/* xr group polls are active */
-			sc_syncbeacon:1,	/* sync/resync beacon timers */
-			sc_hasclrkey:1,		/* CLR key supported */
-			sc_devstopped:1,	/* stopped due to of no tx bufs */
-			sc_stagbeacons:1,	/* use staggered beacons */
-			sc_rtasksched:1, 	/* radar task is scheduled */
-			sc_dfswait:1,    	/* waiting on channel for radar detect */
-			sc_dfstest:1,		/* Test timer in progress */
-		        sc_ackrate:1;           /* send acks at high bitrate */
+
+	unsigned int	sc_invalid:1;		/* being detached */
+	unsigned int	sc_mrretry:1;		/* multi-rate retry support */
+	unsigned int	sc_softled:1;		/* enable LED gpio status */
+	unsigned int	sc_splitmic:1;		/* split TKIP MIC keys */
+	unsigned int	sc_needmib:1;		/* enable MIB stats intr */
+	unsigned int	sc_hasdiversity:1;	/* rx diversity available */
+	unsigned int	sc_diversity:1;		/* enable rx diversity */
+	unsigned int	sc_olddiversity:1;	/* diversity setting before XR enable */
+	unsigned int	sc_hasveol:1;		/* tx VEOL support */
+	unsigned int	sc_hastpc:1;		/* per-packet TPC support */
+	unsigned int	sc_dturbo:1;		/* dynamic turbo capable */
+	unsigned int	sc_dturbo_switch:1;	/* turbo switch mode*/
+	unsigned int	sc_dturbo_hold:1;	/* dynamic turbo hold state */
+	unsigned int	sc_rate_recn_state:1;	/* dynamic turbo state recmded by ratectrl */
+	unsigned int	sc_ignore_ar:1;		/* ignore AR during transition */
+	unsigned int	sc_ledstate:1;		/* LED on/off state */
+	unsigned int	sc_blinking:1;		/* LED blink operation active */
+	unsigned int	sc_beacons:1;		/* beacons running */
+	unsigned int	sc_hasbmask:1;		/* bssid mask support */
+	unsigned int	sc_mcastkey:1;		/* mcast key cache search */
+	unsigned int	sc_hastsfadd:1;		/* tsf adjust support */
+	unsigned int	sc_scanning:1;		/* scanning active */
+	unsigned int	sc_nostabeacons:1;	/* no beacons for station */
+	unsigned int	sc_xrgrppoll:1;		/* xr group polls are active */
+	unsigned int	sc_syncbeacon:1;	/* sync/resync beacon timers */
+	unsigned int	sc_hasclrkey:1;		/* CLR key supported */
+	unsigned int	sc_devstopped:1;	/* stopped due to of no tx bufs */
+	unsigned int	sc_stagbeacons:1;	/* use staggered beacons */
+	unsigned int	sc_rtasksched:1;	/* radar task is scheduled */
+	unsigned int	sc_dfswait:1;		/* waiting on channel for radar detect */
+	unsigned int	sc_dfstest:1;		/* Test timer in progress */
+	unsigned int	sc_ackrate:1;		/* send acks at high bitrate */
+	unsigned int	sc_hasintmit:1;		/* Has interference mitigation */
+	unsigned int	sc_useintmit:1;		/* Use interference mitigation (ANI) */
+
 	/* rate tables */
 	const HAL_RATE_TABLE *sc_rates[IEEE80211_MODE_MAX];
 	const HAL_RATE_TABLE *sc_currates;	/* current rate table */
@@ -655,7 +669,7 @@ struct ath_softc {
 		COMMIT				/* beacon sent, commit change */
 	} sc_updateslot;			/* slot time update fsm */
 	int sc_slotupdate;			/* slot to next advance fsm */
-	struct ieee80211vap *sc_bslot[ATH_BCBUF];/* beacon xmit slots */
+	struct ieee80211vap **sc_bslot;		/* beacon xmit slots */
 	int sc_bnext;				/* next slot for beacon xmit */
 
 	struct timer_list sc_cal_ch;		/* calibration timer */
@@ -1015,5 +1029,11 @@ void ath_sysctl_unregister(void);
 	((*(_ah)->ah_radarWait)((_ah), (_chan)))
 #define ath_hal_get_channel_noise(_ah, _chan) \
 	((*(_ah)->ah_getChanNoise)((_ah), (_chan)))
+#define ath_hal_hasintmit(_ah) \
+	(ath_hal_getcapability(_ah, HAL_CAP_INTMIT, 0, NULL) == HAL_OK)
+#define ath_hal_getintmit(_ah, _dst) \
+	ath_hal_getcapability(_ah, HAL_CAP_INTMIT, 1, _dst)
+#define ath_hal_setintmit(_ah, _v) \
+	ath_hal_setcapability(ah, HAL_CAP_INTMIT, 1, _v, NULL)
 
 #endif /* _DEV_ATH_ATHVAR_H */
